const db = require('../models');
const FileDBApi = require('./file');
const crypto = require('crypto');
const Utils = require('../utils');



const Sequelize = db.Sequelize;
const Op = Sequelize.Op;

module.exports = class MessagesDBApi {
    

    
    static async create(data, options) {
        const currentUser = (options && options.currentUser) || { id: null };
        const transaction = (options && options.transaction) || undefined;

        const messages = await db.messages.create(
            {
                id: data.id || undefined,
        
        message_type: data.message_type
        ||
        null
            ,
            
        body: data.body
        ||
        null
            ,
            
        delivery_status: data.delivery_status
        ||
        null
            ,
            
        sent_at: data.sent_at
        ||
        null
            ,
            
            importHash: data.importHash || null,
            createdById: currentUser.id,
            updatedById: currentUser.id,
    },
        { transaction },
    );

        
        await messages.setConversation( data.conversation || null, {
            transaction,
        });
        
        await messages.setSender( data.sender || null, {
            transaction,
        });
        
        await messages.setReply_to( data.reply_to || null, {
            transaction,
        });
        

        

        
        await FileDBApi.replaceRelationFiles(
            {
                belongsTo: db.messages.getTableName(),
                belongsToColumn: 'images',
                belongsToId: messages.id,
            },
            data.images,
            options,
        );
        
        await FileDBApi.replaceRelationFiles(
            {
                belongsTo: db.messages.getTableName(),
                belongsToColumn: 'attachments',
                belongsToId: messages.id,
            },
            data.attachments,
            options,
        );
        

        return messages;
    }
    

    static async bulkImport(data, options) {
        const currentUser = (options && options.currentUser) || { id: null };
        const transaction = (options && options.transaction) || undefined;

        // Prepare data - wrapping individual data transformations in a map() method
        const messagesData = data.map((item, index) => ({
                id: item.id || undefined,
                
                message_type: item.message_type
            ||
            null
            ,
            
                body: item.body
            ||
            null
            ,
            
                delivery_status: item.delivery_status
            ||
            null
            ,
            
                sent_at: item.sent_at
            ||
            null
            ,
            
            importHash: item.importHash || null,
            createdById: currentUser.id,
            updatedById: currentUser.id,
            createdAt: new Date(Date.now() + index * 1000),
    }));

        // Bulk create items
        const messages = await db.messages.bulkCreate(messagesData, { transaction });

        // For each item created, replace relation files
        
        for (let i = 0; i < messages.length; i++) {
            await FileDBApi.replaceRelationFiles(
                {
                    belongsTo: db.messages.getTableName(),
                    belongsToColumn: 'images',
                    belongsToId: messages[i].id,
                },
                data[i].images,
                options,
            );
        }
        
        for (let i = 0; i < messages.length; i++) {
            await FileDBApi.replaceRelationFiles(
                {
                    belongsTo: db.messages.getTableName(),
                    belongsToColumn: 'attachments',
                    belongsToId: messages[i].id,
                },
                data[i].attachments,
                options,
            );
        }
        

        return messages;
    }

    static async update(id, data,  options) {
        const currentUser = (options && options.currentUser) || {id: null};
        const transaction = (options && options.transaction) || undefined;
        

        const messages = await db.messages.findByPk(id, {}, {transaction});


        

        const updatePayload = {};
        
        if (data.message_type !== undefined) updatePayload.message_type = data.message_type;
        
        
        if (data.body !== undefined) updatePayload.body = data.body;
        
        
        if (data.delivery_status !== undefined) updatePayload.delivery_status = data.delivery_status;
        
        
        if (data.sent_at !== undefined) updatePayload.sent_at = data.sent_at;
        
        
        updatePayload.updatedById = currentUser.id;

        await messages.update(updatePayload, {transaction});

        
        
        if (data.conversation !== undefined) {
            await messages.setConversation(
              
              data.conversation,
              
              { transaction }
            );
        }
        
        if (data.sender !== undefined) {
            await messages.setSender(
              
              data.sender,
              
              { transaction }
            );
        }
        
        if (data.reply_to !== undefined) {
            await messages.setReply_to(
              
              data.reply_to,
              
              { transaction }
            );
        }
        

        
        

        
        await FileDBApi.replaceRelationFiles(
            {
                belongsTo: db.messages.getTableName(),
                belongsToColumn: 'images',
                belongsToId: messages.id,
            },
            data.images,
            options,
        );
        
        await FileDBApi.replaceRelationFiles(
            {
                belongsTo: db.messages.getTableName(),
                belongsToColumn: 'attachments',
                belongsToId: messages.id,
            },
            data.attachments,
            options,
        );
        

        return messages;
    }

    static async deleteByIds(ids, options) {
        const currentUser = (options && options.currentUser) || { id: null };
        const transaction = (options && options.transaction) || undefined;

        const messages = await db.messages.findAll({
            where: {
                id: {
                    [Op.in]: ids,
                },
            },
            transaction,
        });

        await db.sequelize.transaction(async (transaction) => {
            for (const record of messages) {
                await record.update(
                    {deletedBy: currentUser.id},
                    {transaction}
                );
            }
            for (const record of messages) {
                await record.destroy({transaction});
            }
        });


        return messages;
    }

    static async remove(id, options) {
        const currentUser = (options && options.currentUser) || {id: null};
        const transaction = (options && options.transaction) || undefined;

        const messages = await db.messages.findByPk(id, options);

        await messages.update({
            deletedBy: currentUser.id
        }, {
            transaction,
        });

        await messages.destroy({
            transaction
        });

        return messages;
    }

    static async findBy(where, options) {
        const transaction = (options && options.transaction) || undefined;

        const messages = await db.messages.findOne(
            { where },
            { transaction },
        );

        if (!messages) {
            return messages;
        }

        const output = messages.get({plain: true});

        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        output.conversation = await messages.getConversation({
            transaction
        });
        
        
        output.sender = await messages.getSender({
            transaction
        });
        
        
        output.reply_to = await messages.getReply_to({
            transaction
        });
        
        
        output.images = await messages.getImages({
            transaction
        });
        
        
        output.attachments = await messages.getAttachments({
            transaction
        });
        
        

        return output;
    }

    static async findAll(
          filter,
           options
        ) {
        const limit = filter.limit || 0;
        let offset = 0;
        let where = {};
        const currentPage = +filter.page;

        

        

        offset = currentPage * limit;

        const orderBy = null;

    const transaction = (options && options.transaction) || undefined;

    let include = [

      {
        model: db.conversations,
        as: 'conversation',
        
        where: filter.conversation ? {
          [Op.or]: [
            { id: { [Op.in]: filter.conversation.split('|').map(term => Utils.uuid(term)) } },
            {
              title: {
                [Op.or]: filter.conversation.split('|').map(term => ({ [Op.iLike]: `%${term}%` }))
              }
            },
          ]
        } : {},
        
      },

      {
        model: db.users,
        as: 'sender',
        
        where: filter.sender ? {
          [Op.or]: [
            { id: { [Op.in]: filter.sender.split('|').map(term => Utils.uuid(term)) } },
            {
              firstName: {
                [Op.or]: filter.sender.split('|').map(term => ({ [Op.iLike]: `%${term}%` }))
              }
            },
          ]
        } : {},
        
      },

      {
        model: db.messages,
        as: 'reply_to',
        
        where: filter.reply_to ? {
          [Op.or]: [
            { id: { [Op.in]: filter.reply_to.split('|').map(term => Utils.uuid(term)) } },
            {
              body: {
                [Op.or]: filter.reply_to.split('|').map(term => ({ [Op.iLike]: `%${term}%` }))
              }
            },
          ]
        } : {},
        
      },



      {
        model: db.file,
        as: 'images',
      },

      {
        model: db.file,
        as: 'attachments',
      },

    ];

        if (filter) {
            if (filter.id) {
                where = {
                    ...where,
                    ['id']: Utils.uuid(filter.id),
                };
            }

            
            if (filter.body) {
                where = {
                    ...where,
                    [Op.and]: Utils.ilike(
                        'messages',
                        'body',
                        filter.body,
                    ),
                };
            }
            

            
            

            
            if (filter.sent_atRange) {
                const [start, end] = filter.sent_atRange;

                if (start !== undefined && start !== null && start !== '') {
                    where = {
                        ...where,
                    sent_at: {
                    ...where.sent_at,
                            [Op.gte]: start,
                    },
                };
                }

                if (end !== undefined && end !== null && end !== '') {
                    where = {
                        ...where,
                    sent_at: {
                    ...where.sent_at,
                            [Op.lte]: end,
                    },
                };
                }
            }
            

            if (filter.active !== undefined) {
                where = {
                    ...where,
                    active: filter.active === true || filter.active === 'true'
                };
            }

            
            if (filter.message_type) {
                where = {
                    ...where,
                message_type: filter.message_type,
            };
            }
            
            if (filter.delivery_status) {
                where = {
                    ...where,
                delivery_status: filter.delivery_status,
            };
            }
            


      



            if (filter.createdAtRange) {
                const [start, end] = filter.createdAtRange;

                if (start !== undefined && start !== null && start !== '') {
                    where = {
                        ...where,
                        ['createdAt']: {
                            ...where.createdAt,
                            [Op.gte]: start,
                        },
                    };
                }

                if (end !== undefined && end !== null && end !== '') {
                    where = {
                        ...where,
                        ['createdAt']: {
                            ...where.createdAt,
                            [Op.lte]: end,
                        },
                    };
                }
            }
        }
        

        

        const queryOptions = {
            where,
            include,
            distinct: true,
            order: filter.field && filter.sort
                ? [[filter.field, filter.sort]]
                : [['createdAt', 'desc']],
            transaction: options?.transaction,
            logging: console.log
        };

        if (!options?.countOnly) {
            queryOptions.limit = limit ? Number(limit) : undefined;
            queryOptions.offset = offset ? Number(offset) : undefined;
        }

        try {
            const { rows, count } = await db.messages.findAndCountAll(queryOptions);

            return {
                rows: options?.countOnly ? [] : rows,
                count: count
            };
        } catch (error) {
            console.error('Error executing query:', error);
            throw error;
        }
    }

    static async findAllAutocomplete(query, limit, offset, ) {
        let where = {};
        
        

        if (query) {
            where = {
                [Op.or]: [
                    { ['id']: Utils.uuid(query) },
                    Utils.ilike(
                        'messages',
                        'body',
                        query,
                    ),
                ],
            };
        }

        const records = await db.messages.findAll({
            attributes: [ 'id', 'body' ],
            where,
            limit: limit ? Number(limit) : undefined,
            offset: offset ? Number(offset) : undefined,
            orderBy: [['body', 'ASC']],
        });

        return records.map((record) => ({
            id: record.id,
            label: record.body,
        }));
    }

    
};